==> oxygen-gtk-1.1.0.tar.bz2: oxygen-gtk-1.1.0/src/animations/oxygenbackgroundhintengine.h <==
#ifndef oxygenbackgroundhintengine_h
#define oxygenbackgroundhintengine_h
/*
* this file is part of the oxygen gtk engine
* Copyright (c) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
*
* This  library is free  software; you can  redistribute it and/or
* modify it  under  the terms  of the  GNU Lesser  General  Public
* License  as published  by the Free  Software  Foundation; either
* version 2 of the License, or(at your option ) any later version.
*
* This library is distributed  in the hope that it will be useful,
* but  WITHOUT ANY WARRANTY; without even  the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License  along  with  this library;  if not,  write to  the Free
* Software Foundation, Inc., 51  Franklin St, Fifth Floor, Boston,
* MA 02110-1301, USA.
*/

#include "../oxygenflags.h"
#include "oxygenbaseengine.h"

#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <set>

#include <X11/Xdefs.h>

namespace Oxygen
{

    //! forward declaration
    class Animations;
    class StyleHelper;

    enum BackgroundHint
    {

        BackgroundGradient = 1<<0,
        BackgroundPixmap = 1<<1
    };

    OX_DECLARE_FLAGS( BackgroundHints, BackgroundHint )
    OX_DECLARE_OPERATORS_FOR_FLAGS( BackgroundHints )

    //! associates widgets with some type of data
    class BackgroundHintEngine: public BaseEngine
    {

        public:

        //! constructor
        BackgroundHintEngine( Animations* );

        //! destructor
        virtual ~BackgroundHintEngine( void )
        {}

        //! register widget
        virtual bool registerWidget( GtkWidget*, BackgroundHints = BackgroundGradient|BackgroundPixmap );

        //! unregister widget
        virtual void unregisterWidget( GtkWidget* );

        protected:

        //! argb hint atom
        Atom _backgroundGradientAtom;

        //! background gradient hint atom
        Atom _backgroundPixmapAtom;

        //! map widget and window id
        class Data
        {

            public:

            //! constructor
            Data( GtkWidget* widget, XID id ):
                _widget( widget ),
                _id( id )
            {}

            //! equal to operator
            bool operator == (const Data& other ) const
            {
                return
                    other._widget == _widget &&
                    other._id == _id;
            }

            //! less than operator
            bool operator < ( const Data& other ) const
            {
                if( _widget != other._widget ) return _widget < other._widget;
                else return _id < other._id;
            }

            //! widget
            GtkWidget* _widget;

            //! window id
            XID _id;

        };

        //! true if widget is included
        virtual bool contains( const Data& data ) const
        { return _data.find( data ) != _data.end(); }

        //! used to find Data matching given widget
        class SameWidgetFTor
        {

            public:

            //! constructor
            SameWidgetFTor( GtkWidget* widget ):
                _widget( widget )
                {}

            //! predicate
            bool operator () (const Data& data ) const
            { return data._widget == _widget; }

            private:

            //! prediction
            GtkWidget* _widget;

        };

        //! store registered widgets
        std::set<Data> _data;

    };

}

#endif

==> oxygen-gtk2-1.2.2-1.tar.bz2: oxygen-gtk2-1.2.2-1/src/animations/oxygenbackgroundhintengine.h <==
#ifndef oxygenbackgroundhintengine_h
#define oxygenbackgroundhintengine_h
/*
* this file is part of the oxygen gtk engine
* Copyright (c) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
*
* This  library is free  software; you can  redistribute it and/or
* modify it  under  the terms  of the  GNU Lesser  General  Public
* License  as published  by the Free  Software  Foundation; either
* version 2 of the License, or(at your option ) any later version.
*
* This library is distributed  in the hope that it will be useful,
* but  WITHOUT ANY WARRANTY; without even  the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License  along  with  this library;  if not,  write to  the Free
* Software Foundation, Inc., 51  Franklin St, Fifth Floor, Boston,
* MA 02110-1301, USA.
*/

#include "../oxygenflags.h"
#include "oxygenbaseengine.h"

#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <set>
#include <algorithm>

#include <X11/Xdefs.h>

namespace Oxygen
{

    //! forward declaration
    class Animations;
    class StyleHelper;

    enum BackgroundHint
    {

        BackgroundGradient = 1<<0,
        BackgroundPixmap = 1<<1
    };

    OX_DECLARE_FLAGS( BackgroundHints, BackgroundHint )
    OX_DECLARE_OPERATORS_FOR_FLAGS( BackgroundHints )

    //! associates widgets with some type of data
    class BackgroundHintEngine: public BaseEngine
    {

        public:

        //! constructor
        BackgroundHintEngine( Animations* );

        //! destructor
        virtual ~BackgroundHintEngine( void )
        {}

        //! register widget
        virtual bool registerWidget( GtkWidget*, BackgroundHints = BackgroundGradient|BackgroundPixmap );

        //! unregister widget
        virtual void unregisterWidget( GtkWidget* );

        //! returns true if widget is registered
        inline bool contains( GtkWidget* widget ) const;

        protected:

        //! argb hint atom
        Atom _backgroundGradientAtom;

        //! background gradient hint atom
        Atom _backgroundPixmapAtom;

        //! map widget and window id
        class Data
        {

            public:

            //! constructor
            Data( GtkWidget* widget, XID id ):
                _widget( widget ),
                _id( id )
            {}

            //! equal to operator
            bool operator == (const Data& other ) const
            {
                return
                    other._widget == _widget &&
                    other._id == _id;
            }

            //! less than operator
            bool operator < ( const Data& other ) const
            {
                if( _widget != other._widget ) return _widget < other._widget;
                else return _id < other._id;
            }

            //! widget
            GtkWidget* _widget;

            //! window id
            XID _id;

        };

        //! true if widget is included
        virtual bool contains( const Data& data ) const
        { return _data.find( data ) != _data.end(); }

        //! used to find Data matching given widget
        class SameWidgetFTor
        {

            public:

            //! constructor
            SameWidgetFTor( GtkWidget* widget ):
                _widget( widget )
                {}

            //! predicate
            bool operator () (const Data& data ) const
            { return data._widget == _widget; }

            private:

            //! prediction
            GtkWidget* _widget;

        };

        //! store registered widgets
        std::set<Data> _data;

    };

    //____________________________________________________________________
    bool BackgroundHintEngine::contains( GtkWidget* widget ) const
    { return std::find_if( _data.begin(), _data.end(), SameWidgetFTor( widget ) ) != _data.end(); }

}

#endif
